@@ -59,6 +59,20 @@ IRONIC_OPTS = [
                help='Ironic API URL, used to set Ironic API URL when '
                     'auth_strategy option is noauth to work with standalone '
                     'Ironic without keystone.'),
+    cfg.StrOpt('os_service_type',
+               default='baremetal',
+               help='Ironic service type.'),
+    cfg.StrOpt('os_endpoint_type',
+               default='internalURL',
+               help='Ironic endpoint type.'),
+    cfg.IntOpt('retry_interval',
+               default=2,
+               help='Interval between retries in case of conflict error '
+               '(HTTP 409).'),
+    cfg.IntOpt('max_retries',
+               default=30,
+               help='Maximum number of retries in case of conflict error '
+               '(HTTP 409).'),
 ]
 
 
@@ -179,10 +193,6 @@ SERVICE_OPTS = [
                 help='DEPRECATED: use auth_strategy.',
                 deprecated_group='discoverd',
                 deprecated_for_removal=True),
-    cfg.BoolOpt('debug',
-                default=False,
-                help='Debug mode enabled/disabled.',
-                deprecated_group='discoverd'),
     cfg.IntOpt('timeout',
                default=3600,
                help='Timeout after which introspection is considered failed, '
@@ -210,7 +220,14 @@ SERVICE_OPTS = [
                help='Path to SSL key'),
     cfg.IntOpt('max_concurrency',
                default=1000,
-               help='The green thread pool size.')
+               help='The green thread pool size.'),
+    cfg.IntOpt('introspection_delay',
+               default=5,
+               help='Delay (in seconds) between two introspections.'),
+    cfg.StrOpt('introspection_delay_drivers',
+               default='^.*_ssh$',
+               help='Only node with drivers matching this regular expression '
+               'will be affected by introspection_delay setting.'),
 ]
 
 
