@@ -19,6 +19,7 @@ import unittest
 import mock
 from oslo_utils import uuidutils
 
+from ironic_inspector import db
 from ironic_inspector import firewall
 from ironic_inspector import introspect
 from ironic_inspector import main
@@ -33,21 +34,28 @@ from oslo_config import cfg
 CONF = cfg.CONF
 
 
-class TestApi(test_base.BaseTest):
+def _get_error(res):
+    return json.loads(res.data.decode('utf-8'))['error']['message']
+
+
+class BaseAPITest(test_base.BaseTest):
     def setUp(self):
-        super(TestApi, self).setUp()
+        super(BaseAPITest, self).setUp()
         main.app.config['TESTING'] = True
         self.app = main.app.test_client()
         CONF.set_override('auth_strategy', 'noauth')
         self.uuid = uuidutils.generate_uuid()
 
+
+class TestApiIntrospect(BaseAPITest):
     @mock.patch.object(introspect, 'introspect', autospec=True)
     def test_introspect_no_authentication(self, introspect_mock):
         CONF.set_override('auth_strategy', 'noauth')
         res = self.app.post('/v1/introspection/%s' % self.uuid)
         self.assertEqual(202, res.status_code)
         introspect_mock.assert_called_once_with(self.uuid,
-                                                new_ipmi_credentials=None)
+                                                new_ipmi_credentials=None,
+                                                token=None)
 
     @mock.patch.object(introspect, 'introspect', autospec=True)
     def test_introspect_set_ipmi_credentials(self, introspect_mock):
@@ -56,7 +64,8 @@ class TestApi(test_base.BaseTest):
         self.assertEqual(202, res.status_code)
         introspect_mock.assert_called_once_with(
             self.uuid,
-            new_ipmi_credentials=('user', 'password'))
+            new_ipmi_credentials=('user', 'password'),
+            token=None)
 
     @mock.patch.object(introspect, 'introspect', autospec=True)
     def test_introspect_set_ipmi_credentials_no_user(self, introspect_mock):
@@ -65,7 +74,8 @@ class TestApi(test_base.BaseTest):
         self.assertEqual(202, res.status_code)
         introspect_mock.assert_called_once_with(
             self.uuid,
-            new_ipmi_credentials=(None, 'password'))
+            new_ipmi_credentials=(None, 'password'),
+            token=None)
 
     @mock.patch.object(introspect, 'introspect', autospec=True)
     def test_intospect_failed(self, introspect_mock):
@@ -77,7 +87,8 @@ class TestApi(test_base.BaseTest):
             json.loads(res.data.decode('utf-8'))['error']['message'])
         introspect_mock.assert_called_once_with(
             self.uuid,
-            new_ipmi_credentials=None)
+            new_ipmi_credentials=None,
+            token=None)
 
     @mock.patch.object(utils, 'check_auth', autospec=True)
     @mock.patch.object(introspect, 'introspect', autospec=True)
@@ -96,6 +107,8 @@ class TestApi(test_base.BaseTest):
         res = self.app.post('/v1/introspection/%s' % uuid_dummy)
         self.assertEqual(400, res.status_code)
 
+
+class TestApiContinue(BaseAPITest):
     @mock.patch.object(process, 'process', autospec=True)
     def test_continue(self, process_mock):
         # should be ignored
@@ -112,10 +125,10 @@ class TestApi(test_base.BaseTest):
         res = self.app.post('/v1/continue', data='"JSON"')
         self.assertEqual(400, res.status_code)
         process_mock.assert_called_once_with("JSON")
-        self.assertEqual(
-            'boom',
-            json.loads(res.data.decode('utf-8'))['error']['message'])
+        self.assertEqual('boom', _get_error(res))
 
+
+class TestApiGetStatus(BaseAPITest):
     @mock.patch.object(node_cache, 'get_node', autospec=True)
     def test_get_introspection_in_progress(self, get_mock):
         get_mock.return_value = node_cache.NodeInfo(uuid=self.uuid,
@@ -137,6 +150,85 @@ class TestApi(test_base.BaseTest):
                          json.loads(res.data.decode('utf-8')))
 
 
+class TestApiMisc(BaseAPITest):
+    @mock.patch.object(node_cache, 'get_node', autospec=True)
+    def test_404_expected(self, get_mock):
+        get_mock.side_effect = iter([utils.Error('boom', code=404)])
+        res = self.app.get('/v1/introspection/%s' % self.uuid)
+        self.assertEqual(404, res.status_code)
+        self.assertEqual('boom', _get_error(res))
+
+    def test_404_unexpected(self):
+        res = self.app.get('/v42')
+        self.assertEqual(404, res.status_code)
+        self.assertIn('not found', _get_error(res).lower())
+
+    @mock.patch.object(node_cache, 'get_node', autospec=True)
+    def test_500_with_debug(self, get_mock):
+        CONF.set_override('debug', True)
+        get_mock.side_effect = iter([RuntimeError('boom')])
+        res = self.app.get('/v1/introspection/%s' % self.uuid)
+        self.assertEqual(500, res.status_code)
+        self.assertEqual('Internal server error (RuntimeError): boom',
+                         _get_error(res))
+
+    @mock.patch.object(node_cache, 'get_node', autospec=True)
+    def test_500_without_debug(self, get_mock):
+        CONF.set_override('debug', False)
+        get_mock.side_effect = iter([RuntimeError('boom')])
+        res = self.app.get('/v1/introspection/%s' % self.uuid)
+        self.assertEqual(500, res.status_code)
+        self.assertEqual('Internal server error',
+                         _get_error(res))
+
+
+class TestApiVersions(BaseAPITest):
+    def _check_version_present(self, res):
+        self.assertEqual('%d.%d' % main.MINIMUM_API_VERSION,
+                         res.headers.get(main._MIN_VERSION_HEADER))
+        self.assertEqual('%d.%d' % main.CURRENT_API_VERSION,
+                         res.headers.get(main._MAX_VERSION_HEADER))
+
+    def test_root_endpoints(self):
+        for endpoint in '/', '/v1':
+            res = self.app.get(endpoint)
+            self.assertEqual(200, res.status_code)
+            self._check_version_present(res)
+
+    def test_404_unexpected(self):
+        # API version on unknown pages
+        self._check_version_present(self.app.get('/v1/foobar'))
+
+    @mock.patch.object(node_cache, 'get_node', autospec=True)
+    def test_usual_requests(self, get_mock):
+        get_mock.return_value = node_cache.NodeInfo(uuid=self.uuid,
+                                                    started_at=42.0)
+        # Successfull
+        self._check_version_present(
+            self.app.post('/v1/introspection/%s' % self.uuid))
+        # With error
+        self._check_version_present(
+            self.app.post('/v1/introspection/foobar'))
+
+    def test_request_correct_version(self):
+        headers = {main._VERSION_HEADER:
+                   main._format_version(main.CURRENT_API_VERSION)}
+        self._check_version_present(self.app.get('/', headers=headers))
+
+    def test_request_unsupported_version(self):
+        bad_version = (main.CURRENT_API_VERSION[0],
+                       main.CURRENT_API_VERSION[1] + 1)
+        headers = {main._VERSION_HEADER:
+                   main._format_version(bad_version)}
+        res = self.app.get('/', headers=headers)
+        self._check_version_present(res)
+        self.assertEqual(406, res.status_code)
+        error = _get_error(res)
+        self.assertIn('%d.%d' % bad_version, error)
+        self.assertIn('%d.%d' % main.MINIMUM_API_VERSION, error)
+        self.assertIn('%d.%d' % main.CURRENT_API_VERSION, error)
+
+
 class TestPlugins(unittest.TestCase):
     @mock.patch.object(example_plugin.ExampleProcessingHook,
                        'before_processing', autospec=True)
@@ -160,7 +252,7 @@ class TestPlugins(unittest.TestCase):
 @mock.patch.object(firewall, 'init')
 @mock.patch.object(utils, 'add_auth_middleware')
 @mock.patch.object(utils, 'get_client')
-@mock.patch.object(node_cache, 'init')
+@mock.patch.object(db, 'init')
 class TestInit(test_base.BaseTest):
     def test_ok(self, mock_node_cache, mock_get_client, mock_auth,
                 mock_firewall, mock_spawn_n):
