@@ -11,13 +11,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import json
 import time
 import unittest
 
 import mock
 from oslo_config import cfg
 
-from ironic_inspector import models
+from ironic_inspector import db
 from ironic_inspector import node_cache
 from ironic_inspector.test import base as test_base
 from ironic_inspector import utils
@@ -28,72 +29,137 @@ CONF = cfg.CONF
 class TestNodeCache(test_base.NodeTest):
     def test_add_node(self):
         # Ensure previous node information is cleared
-        session = node_cache.get_session()
+        session = db.get_session()
         with session.begin():
-            models.Node(uuid=self.node.uuid).save(session)
-            models.Node(uuid='uuid2').save(session)
-            models.Attribute(name='mac',
-                             value='11:22:11:22:11:22',
-                             uuid=self.uuid).save(session)
+            db.Node(uuid=self.node.uuid).save(session)
+            db.Node(uuid='uuid2').save(session)
+            db.Attribute(name='mac',
+                         value='11:22:11:22:11:22',
+                         uuid=self.uuid).save(session)
 
         res = node_cache.add_node(self.node.uuid, mac=self.macs,
                                   bmc_address='1.2.3.4', foo=None)
         self.assertEqual(self.uuid, res.uuid)
         self.assertTrue(time.time() - 60 < res.started_at < time.time() + 60)
 
-        res = (node_cache.model_query(models.Node.uuid,
-               models.Node.started_at).order_by(models.Node.uuid).all())
+        res = (db.model_query(db.Node.uuid,
+               db.Node.started_at).order_by(db.Node.uuid).all())
         self.assertEqual(['1a1a1a1a-2b2b-3c3c-4d4d-5e5e5e5e5e5e',
                           'uuid2'], [t.uuid for t in res])
         self.assertTrue(time.time() - 60 < res[0].started_at <
                         time.time() + 60)
 
-        res = (node_cache.model_query(models.Attribute.name,
-               models.Attribute.value, models.Attribute.uuid).
-               order_by(models.Attribute.name, models.Attribute.value).all())
+        res = (db.model_query(db.Attribute.name,
+                              db.Attribute.value, db.Attribute.uuid).
+               order_by(db.Attribute.name, db.Attribute.value).all())
         self.assertEqual([('bmc_address', '1.2.3.4', self.uuid),
                           ('mac', self.macs[0], self.uuid),
                           ('mac', self.macs[1], self.uuid)],
                          [(row.name, row.value, row.uuid) for row in res])
 
-    def test_add_node_duplicate_mac(self):
-        session = node_cache.get_session()
+    def test__delete_node(self):
+        session = db.get_session()
         with session.begin():
-            models.Node(uuid='another-uuid').save(session)
-            models.Attribute(name='mac', value='11:22:11:22:11:22',
-                             uuid='another-uuid').save(session)
+            db.Node(uuid=self.node.uuid).save(session)
+            db.Attribute(name='mac', value='11:22:11:22:11:22',
+                         uuid=self.uuid).save(session)
+            data = {'s': 'value', 'b': True, 'i': 42}
+            encoded = json.dumps(data)
+            db.Option(uuid=self.uuid, name='name', value=encoded).save(
+                session)
+
+        node_cache._delete_node(self.uuid)
+        session = db.get_session()
+        row_node = db.model_query(db.Node).filter_by(
+            uuid=self.uuid).first()
+        self.assertIsNone(row_node)
+        row_attribute = db.model_query(db.Attribute).filter_by(
+            uuid=self.uuid).first()
+        self.assertIsNone(row_attribute)
+        row_option = db.model_query(db.Option).filter_by(
+            uuid=self.uuid).first()
+        self.assertIsNone(row_option)
+
+    @mock.patch.object(node_cache, '_list_node_uuids')
+    @mock.patch.object(node_cache, '_delete_node')
+    def test_delete_nodes_not_in_list(self, mock__delete_node,
+                                      mock__list_node_uuids):
+        uuid2 = 'uuid2'
+        uuids = {self.uuid}
+        mock__list_node_uuids.return_value = {self.uuid, uuid2}
+        session = db.get_session()
+        with session.begin():
+            node_cache.delete_nodes_not_in_list(uuids)
+        mock__delete_node.assert_called_once_with(uuid2)
+
+    def test_add_node_duplicate_mac(self):
+        session = db.get_session()
+        with session.begin():
+            db.Node(uuid='another-uuid').save(session)
+            db.Attribute(name='mac', value='11:22:11:22:11:22',
+                         uuid='another-uuid').save(session)
         self.assertRaises(utils.Error,
                           node_cache.add_node,
                           self.node.uuid, mac=['11:22:11:22:11:22'])
 
     def test_active_macs(self):
-        session = node_cache.get_session()
+        session = db.get_session()
         with session.begin():
-            models.Node(uuid=self.node.uuid).save(session)
+            db.Node(uuid=self.node.uuid).save(session)
             values = [('mac', '11:22:11:22:11:22', self.uuid),
                       ('mac', '22:11:22:11:22:11', self.uuid)]
             for value in values:
-                models.Attribute(name=value[0], value=value[1],
-                                 uuid=value[2]).save(session)
+                db.Attribute(name=value[0], value=value[1],
+                             uuid=value[2]).save(session)
         self.assertEqual({'11:22:11:22:11:22', '22:11:22:11:22:11'},
                          node_cache.active_macs())
 
-    def test_add_attribute(self):
-        session = node_cache.get_session()
+    def test__list_node_uuids(self):
+        session = db.get_session()
         with session.begin():
-            models.Node(uuid=self.node.uuid).save(session)
+            db.Node(uuid=self.node.uuid).save(session)
+            db.Node(uuid='uuid2').save(session)
+
+        node_uuid_list = node_cache._list_node_uuids()
+        self.assertEqual({self.uuid, 'uuid2'}, node_uuid_list)
+
+    def test_add_attribute(self):
+        session = db.get_session()
+        with session.begin():
+            db.Node(uuid=self.node.uuid).save(session)
         node_info = node_cache.NodeInfo(uuid=self.uuid, started_at=42)
         node_info.add_attribute('key', 'value')
-        res = node_cache.model_query(models.Attribute.name,
-                                     models.Attribute.value,
-                                     models.Attribute.uuid,
-                                     session=session).order_by(
-                                         models.Attribute.name,
-                                         models.Attribute.value).all()
+        res = db.model_query(db.Attribute.name,
+                             db.Attribute.value,
+                             db.Attribute.uuid,
+                             session=session)
+        res = res.order_by(db.Attribute.name, db.Attribute.value).all()
         self.assertEqual([('key', 'value', self.uuid)],
                          [tuple(row) for row in res])
         self.assertRaises(utils.Error, node_info.add_attribute,
                           'key', 'value')
+        # check that .attributes got invalidated and reloaded
+        self.assertEqual({'key': ['value']}, node_info.attributes)
+
+    def test_attributes(self):
+        node_info = node_cache.add_node(self.uuid,
+                                        bmc_address='1.2.3.4',
+                                        mac=self.macs)
+        self.assertEqual({'bmc_address': ['1.2.3.4'],
+                          'mac': self.macs},
+                         node_info.attributes)
+        # check invalidation
+        session = db.get_session()
+        with session.begin():
+            db.Attribute(name='foo', value='bar', uuid=self.uuid).save(session)
+        # still cached
+        self.assertEqual({'bmc_address': ['1.2.3.4'],
+                          'mac': self.macs},
+                         node_info.attributes)
+        node_info.invalidate_cache()
+        self.assertEqual({'bmc_address': ['1.2.3.4'],
+                          'mac': self.macs, 'foo': ['bar']},
+                         node_info.attributes)
 
 
 class TestNodeCacheFind(test_base.NodeTest):
@@ -135,17 +201,17 @@ class TestNodeCacheFind(test_base.NodeTest):
         self.assertTrue(time.time() - 60 < res.started_at < time.time() + 1)
 
     def test_inconsistency(self):
-        session = node_cache.get_session()
+        session = db.get_session()
         with session.begin():
-            (node_cache.model_query(models.Node).filter_by(uuid=self.uuid).
+            (db.model_query(db.Node).filter_by(uuid=self.uuid).
                 delete())
         self.assertRaises(utils.Error, node_cache.find_node,
                           bmc_address='1.2.3.4')
 
     def test_already_finished(self):
-        session = node_cache.get_session()
+        session = db.get_session()
         with session.begin():
-            (node_cache.model_query(models.Node).filter_by(uuid=self.uuid).
+            (db.model_query(db.Node).filter_by(uuid=self.uuid).
                 update({'finished_at': 42.0}))
         self.assertRaises(utils.Error, node_cache.find_node,
                           bmc_address='1.2.3.4')
@@ -155,14 +221,14 @@ class TestNodeCacheCleanUp(test_base.NodeTest):
     def setUp(self):
         super(TestNodeCacheCleanUp, self).setUp()
         self.started_at = 100.0
-        session = node_cache.get_session()
+        session = db.get_session()
         with session.begin():
-            models.Node(uuid=self.uuid, started_at=self.started_at).save(
+            db.Node(uuid=self.uuid, started_at=self.started_at).save(
                 session)
             for v in self.macs:
-                models.Attribute(name='mac', value=v, uuid=self.uuid).save(
+                db.Attribute(name='mac', value=v, uuid=self.uuid).save(
                     session)
-            models.Option(uuid=self.uuid, name='foo', value='bar').save(
+            db.Option(uuid=self.uuid, name='foo', value='bar').save(
                 session)
 
     def test_no_timeout(self):
@@ -171,12 +237,12 @@ class TestNodeCacheCleanUp(test_base.NodeTest):
         self.assertFalse(node_cache.clean_up())
 
         res = [tuple(row) for row in
-               node_cache.model_query(models.Node.finished_at,
-               models.Node.error).all()]
+               db.model_query(db.Node.finished_at,
+                              db.Node.error).all()]
         self.assertEqual([(None, None)], res)
         self.assertEqual(len(self.macs),
-                         node_cache.model_query(models.Attribute).count())
-        self.assertEqual(1, node_cache.model_query(models.Option).count())
+                         db.model_query(db.Attribute).count())
+        self.assertEqual(1, db.model_query(db.Option).count())
 
     @mock.patch.object(time, 'time')
     def test_ok(self, time_mock):
@@ -184,52 +250,52 @@ class TestNodeCacheCleanUp(test_base.NodeTest):
 
         self.assertFalse(node_cache.clean_up())
 
-        res = [tuple(row) for row in node_cache.model_query(
-            models.Node.finished_at, models.Node.error).all()]
+        res = [tuple(row) for row in db.model_query(
+            db.Node.finished_at, db.Node.error).all()]
         self.assertEqual([(None, None)], res)
         self.assertEqual(len(self.macs),
-                         node_cache.model_query(models.Attribute).count())
-        self.assertEqual(1, node_cache.model_query(models.Option).count())
+                         db.model_query(db.Attribute).count())
+        self.assertEqual(1, db.model_query(db.Option).count())
 
     @mock.patch.object(time, 'time')
     def test_timeout(self, time_mock):
         # Add a finished node to confirm we don't try to timeout it
         time_mock.return_value = self.started_at
-        session = node_cache.get_session()
+        session = db.get_session()
         with session.begin():
-            models.Node(uuid=self.uuid + '1', started_at=self.started_at,
-                        finished_at=self.started_at + 60).save(session)
+            db.Node(uuid=self.uuid + '1', started_at=self.started_at,
+                    finished_at=self.started_at + 60).save(session)
         CONF.set_override('timeout', 99)
         time_mock.return_value = (self.started_at + 100)
 
         self.assertEqual([self.uuid], node_cache.clean_up())
 
         res = [(row.finished_at, row.error) for row in
-               node_cache.model_query(models.Node).all()]
+               db.model_query(db.Node).all()]
         self.assertEqual([(self.started_at + 100, 'Introspection timeout'),
                           (self.started_at + 60, None)],
                          res)
-        self.assertEqual([], node_cache.model_query(models.Attribute).all())
-        self.assertEqual([], node_cache.model_query(models.Option).all())
+        self.assertEqual([], db.model_query(db.Attribute).all())
+        self.assertEqual([], db.model_query(db.Option).all())
 
     def test_old_status(self):
         CONF.set_override('node_status_keep_time', 42)
-        session = node_cache.get_session()
+        session = db.get_session()
         with session.begin():
-            node_cache.model_query(models.Node).update(
+            db.model_query(db.Node).update(
                 {'finished_at': time.time() - 100})
 
         self.assertEqual([], node_cache.clean_up())
 
-        self.assertEqual([], node_cache.model_query(models.Node).all())
+        self.assertEqual([], db.model_query(db.Node).all())
 
 
 class TestNodeCacheGetNode(test_base.NodeTest):
     def test_ok(self):
         started_at = time.time() - 42
-        session = node_cache.get_session()
+        session = db.get_session()
         with session.begin():
-            models.Node(uuid=self.uuid, started_at=started_at).save(session)
+            db.Node(uuid=self.uuid, started_at=started_at).save(session)
         info = node_cache.get_node(self.uuid)
 
         self.assertEqual(self.uuid, info.uuid)
@@ -249,33 +315,33 @@ class TestNodeInfoFinished(test_base.NodeTest):
                             bmc_address='1.2.3.4',
                             mac=self.macs)
         self.node_info = node_cache.NodeInfo(uuid=self.uuid, started_at=3.14)
-        session = node_cache.get_session()
+        session = db.get_session()
         with session.begin():
-            models.Option(uuid=self.uuid, name='foo', value='bar').save(
+            db.Option(uuid=self.uuid, name='foo', value='bar').save(
                 session)
 
     def test_success(self):
         self.node_info.finished()
 
-        session = node_cache.get_session()
+        session = db.get_session()
         with session.begin():
             self.assertEqual((42.0, None),
-                             tuple(node_cache.model_query(
-                                   models.Node.finished_at,
-                                   models.Node.error).first()))
-            self.assertEqual([], node_cache.model_query(models.Attribute,
+                             tuple(db.model_query(
+                                   db.Node.finished_at,
+                                   db.Node.error).first()))
+            self.assertEqual([], db.model_query(db.Attribute,
                              session=session).all())
-            self.assertEqual([], node_cache.model_query(models.Option,
+            self.assertEqual([], db.model_query(db.Option,
                              session=session).all())
 
     def test_error(self):
         self.node_info.finished(error='boom')
 
         self.assertEqual((42.0, 'boom'),
-                         tuple(node_cache.model_query(models.Node.finished_at,
-                               models.Node.error).first()))
-        self.assertEqual([], node_cache.model_query(models.Attribute).all())
-        self.assertEqual([], node_cache.model_query(models.Option).all())
+                         tuple(db.model_query(db.Node.finished_at,
+                               db.Node.error).first()))
+        self.assertEqual([], db.model_query(db.Attribute).all())
+        self.assertEqual([], db.model_query(db.Option).all())
 
 
 class TestInit(unittest.TestCase):
@@ -283,9 +349,9 @@ class TestInit(unittest.TestCase):
         super(TestInit, self).setUp()
 
     def test_ok(self):
-        node_cache.init()
-        session = node_cache.get_session()
-        node_cache.model_query(models.Node, session=session)
+        db.init()
+        session = db.get_session()
+        db.model_query(db.Node, session=session)
 
 
 class TestNodeInfoOptions(test_base.NodeTest):
@@ -295,9 +361,9 @@ class TestNodeInfoOptions(test_base.NodeTest):
                             bmc_address='1.2.3.4',
                             mac=self.macs)
         self.node_info = node_cache.NodeInfo(uuid=self.uuid, started_at=3.14)
-        session = node_cache.get_session()
+        session = db.get_session()
         with session.begin():
-            models.Option(uuid=self.uuid, name='foo', value='"bar"').save(
+            db.Option(uuid=self.uuid, name='foo', value='"bar"').save(
                 session)
 
     def test_get(self):
