@@ -443,34 +443,6 @@ class TestProcessNode(BaseTest):
 
         self.cli.node.update.assert_called_once_with(self.uuid, patch)
 
-    def test_update_retry_on_conflict(self, filters_mock, post_hook_mock):
-        self.cli.node.update.side_effect = [exceptions.Conflict, self.node,
-                                            exceptions.Conflict, self.node]
-
-        self.call()
-
-        self.cli.port.create.assert_any_call(node_uuid=self.uuid,
-                                             address=self.macs[0])
-        self.cli.port.create.assert_any_call(node_uuid=self.uuid,
-                                             address=self.macs[1])
-        self.cli.node.update.assert_called_with(self.uuid, self.patch_props)
-        self.assertEqual(2, self.cli.node.update.call_count)
-        self.cli.node.set_power_state.assert_called_once_with(self.uuid, 'off')
-
-    def test_power_off_retry_on_conflict(self, filters_mock, post_hook_mock):
-        self.cli.node.set_power_state.side_effect = [exceptions.Conflict, None]
-
-        self.call()
-
-        self.cli.port.create.assert_any_call(node_uuid=self.uuid,
-                                             address=self.macs[0])
-        self.cli.port.create.assert_any_call(node_uuid=self.uuid,
-                                             address=self.macs[1])
-        self.cli.node.update.assert_called_once_with(self.uuid,
-                                                     self.patch_props)
-        self.cli.node.set_power_state.assert_called_with(self.uuid, 'off')
-        self.assertEqual(2, self.cli.node.set_power_state.call_count)
-
     def test_port_failed(self, filters_mock, post_hook_mock):
         self.cli.port.create.side_effect = (
             [exceptions.Conflict()] + self.ports[1:])
